/** @file
 *
 * @defgroup looptimus_plus looptimus_app.c
 * @{
 * @brief    main functionality of Looptimus Plus device.
 *
 * This file contains the source code for application that looptimus plus
 * device requires to do.
 * 
 */
#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>
#include <string.h>

#include "definition.h"

//#define STANDARD_VALUE

boolean gBatLow = false;
extern int xBLE_mode;
extern bool gIsExternalPowerExists;
boolean gLowBatVoltageIndication = false;
extern byte gMode_midi;
/* Declaration of boot functions*/
void BootModeRST(void);
void BootModeExp (void);

#define  BOTTON_PRESS_TIME          2000
       
#define millis xTaskGetTickCount

#define EXTERNAL_SUPPLY_IS_CONNECTED

int8_t short_btnpress[ MINI_BUTTONS] = {false, false, false};
int8_t long_btnpress[MINI_BUTTONS] =  {false, false, false};
byte miniButtonPressed[MINI_BUTTONS] = {false, false, false};
byte keyboard_button_pressed[MINI_BUTTONS] = {false, false, false};

long miniBtnTimeDown[MINI_BUTTONS] = {0, 0, 0};
byte miniButtonFalling[MINI_BUTTONS] = {false, false, false};

long tCurrentTime = 0;
long tElapsedTime = 0;
long tRequiredTime = 0;
long tick150ms = 0;
long tick2000ms = 0;

long stopTime = 0;  //Time the stop button was pressed

boolean  expNext = 0;
byte expValMapped = -1;
int expRead = 0;
int expVal = 0;
int lastExpVal = 0;
int lastExpVal2 = 0;
int8_t lastExpValMapped = -1;
long lastExpTime = 0;
int expLow = EXPLOWDEF;  //Current low end cutoff
int expHigh = EXPHIGHDEF;  //Current high end cutoff
int8_t config_midi_mode;
int8_t config_keyboard_mode;
extern byte gMode_HID_Keyboard;
static uint16_t firstTime = 0;

extern int g_batvolt;
/*user defined functions*/
void Check_BAT_Voltage();
void Process_midi_Buttons(void);
void ext_pedal_process(void);

/* MiDi Flags */
typedef struct 
{
  bool  poc_test_done;
  bool  noteModeFlag;
  bool  expModeFlag;
  bool  pcModeFlag;
  bool  ccModeFlag;
  bool rstModeFlag;

  bool detect_ext_pedal;
}MIDI_FLAGS;

/* Keyboard Flags */
typedef struct 
{
  bool hid_rstModeFlag;
  bool  keyboard_mode_A_Flag;
  bool  keyboard_mode_B_Flag;
  bool  keyboard_mode_C_Flag;
}KEYBOARD_HID_FLAGS;

/* Keyboard button press flag*/
typedef struct
{
  bool keyboard_button_A_Detected;
  bool keyboard_button_B_Detected;
  bool keyboard_button_C_Detected;
}KEYBOARD_HID_BUTTONS_DETECTION;

/* Looptimus Variables */
typedef struct 
{
  int gread_button_value;
  long hold_time;
}LOOP_VAR;

static MIDI_FLAGS midi_flags;
static KEYBOARD_HID_FLAGS keyboard_HID_flag;
static KEYBOARD_HID_BUTTONS_DETECTION keyboard_button_status;
static LOOP_VAR loop_variable;
MAIN_MIDI_MODE_SET midi_mode_set;
MAIN_HID_KEYBOARD_MODE_SET keyboard_mode_set;

bool set_exp_pedal_button_detect = false;

byte button_long_detect[3] = { false, false, false};

/* Expression Pedal*/
#define filterSamples   23              // filterSamples should  be an odd number, no smaller than 3

#define max(a, b) ( ((a) > (b))? (a) : (b))
#define min(a, b) ( ((a) < (b))? (a) : (b))

int expSmoothArray [filterSamples];   // array for holding raw sensor values for sensor1 

void g_sent_Notes(byte x_note, byte y_velocity, byte z_channel);

/* ---------------------- Check On-Board Button Status ------------------------*/
void Check_button_status(int gadc_button_value)
{
    /*
    With Battery
    G: 460 to 711 > 1.6172 to 2.4996
    H: 220 to 345 > 0.7734 to 1.2129
    I: 114 to 180 > 0.4008 to 0.6328

    With external supply
    G: 460 + 320 to 711 + 140 > 780 to 851 > 2.7422 to 2.9918
    H: 220 + 175 to 345 +  85 > 395 to 430 > 1.3887 to 1.5117
    I: 114 +  62 to 180 +  40 > 176 to 220 > 0.6187 to 0.7734

    */
    // ADC Sampling on Battery 
    uint16_t tA_BTN_LWR_RANGE_LIMIT = 460, tA_BTN_UPR_RANGE_LIMIT = 711;
    uint16_t tB_BTN_LWR_RANGE_LIMIT = 220, tB_BTN_UPR_RANGE_LIMIT = 345;
    uint16_t tC_BTN_LWR_RANGE_LIMIT = 114, tC_BTN_UPR_RANGE_LIMIT = 180;

    if(gIsExternalPowerExists)
    {
#ifdef  EXTERNAL_SUPPLY_IS_CONNECTED
            //ADC Sampling on External Power
            tA_BTN_LWR_RANGE_LIMIT += 320; tA_BTN_UPR_RANGE_LIMIT += 140;
            tB_BTN_LWR_RANGE_LIMIT += 175; tB_BTN_UPR_RANGE_LIMIT += 85;
            tC_BTN_LWR_RANGE_LIMIT += 62;  tC_BTN_UPR_RANGE_LIMIT += 40;
#endif
    }



    //check button status
    if(gadc_button_value < 100) //button is not pressed
    {
           //uint16_t tCurrentTime = millis();
            tCurrentTime = millis();
            if (miniButtonFalling[MINI_BUTTON_A] == true)
            { 
                miniButtonFalling[MINI_BUTTON_A] = false;                
                tElapsedTime = (tCurrentTime - miniBtnTimeDown[MINI_BUTTON_A]);
                tick150ms = pdMS_TO_TICKS(150);
                tick2000ms = pdMS_TO_TICKS(1000);

                if ((tElapsedTime >= tick150ms) && 
                    (tElapsedTime < tick2000ms) )
                {
                    //bsp_board_led_invert(BLE_LED_NUM);
                    miniButtonPressed[MINI_BUTTON_A] = true;
                }

                else if (long_btnpress[MINI_BUTTON_A])
                {
                      long_btnpress[MINI_BUTTON_A] = false;
                      button_long_detect[MINI_BUTTON_A] = true;
                
                }
                miniBtnTimeDown[MINI_BUTTON_A] = 0;
            }
            else if (miniButtonFalling[MINI_BUTTON_B] == true)
            {
                miniButtonFalling[MINI_BUTTON_B] = false;
                tElapsedTime = (tCurrentTime - miniBtnTimeDown[MINI_BUTTON_B]);
                if ((tElapsedTime >= pdMS_TO_TICKS(150)) && 
                    (tElapsedTime < pdMS_TO_TICKS(1000)))
                {
                    miniButtonPressed[MINI_BUTTON_B] = true;
                }
                else if (long_btnpress[MINI_BUTTON_B])
                {
                      long_btnpress[MINI_BUTTON_B] = false;
                      button_long_detect[MINI_BUTTON_B] = true;
                
                }
                miniBtnTimeDown[MINI_BUTTON_B] = 0;
            }
            else if (miniButtonFalling[MINI_BUTTON_C] == true)
            {
                miniButtonFalling[MINI_BUTTON_C] = false;
                tElapsedTime = (tCurrentTime - miniBtnTimeDown[MINI_BUTTON_C]);
                if ((tElapsedTime >= pdMS_TO_TICKS(150)) && 
                    (tElapsedTime < pdMS_TO_TICKS(1000)) )
                {
                    miniButtonPressed[MINI_BUTTON_C] = true;
                    set_exp_pedal_button_detect = true;

                }

                else if (long_btnpress[MINI_BUTTON_C])
                {
                      long_btnpress[MINI_BUTTON_C] = false;
                      button_long_detect[MINI_BUTTON_C] = true;
                
                }
                miniBtnTimeDown[MINI_BUTTON_C] = 0;
            }            
    }    
    else if (gadc_button_value > 100) //button is pressed
    {
    
        if ((tA_BTN_LWR_RANGE_LIMIT < gadc_button_value) && (gadc_button_value < tA_BTN_UPR_RANGE_LIMIT))
          {
                // "A" button press detected
                if (miniBtnTimeDown[MINI_BUTTON_A] == 0)
                {
                    miniButtonFalling[MINI_BUTTON_A] = true;
                    miniBtnTimeDown[MINI_BUTTON_A] = millis();
                }
                else
                {
                    tCurrentTime = millis();
                    if ((tCurrentTime - miniBtnTimeDown[MINI_BUTTON_A]) >= pdMS_TO_TICKS(BSP_LONG_PUSH_TIMEOUT_MS))
                    {
                        // "A" button kept pressed for more than 2 seconds
                        long_btnpress[MINI_BUTTON_A] = true;
                    }
                }
            }
            else if ((tB_BTN_LWR_RANGE_LIMIT < gadc_button_value) && (gadc_button_value < tB_BTN_UPR_RANGE_LIMIT))
            {
                // "B" button press detected
                if (miniBtnTimeDown[MINI_BUTTON_B] == 0)
                {
                    miniButtonFalling[MINI_BUTTON_B] = true;
                    miniBtnTimeDown[MINI_BUTTON_B] = millis();
                }
                else
                {
                    tCurrentTime = millis();
                    if ((tCurrentTime - miniBtnTimeDown[MINI_BUTTON_B]) >= pdMS_TO_TICKS(BSP_LONG_PUSH_TIMEOUT_MS))
                    {
                        // "B" button kept pressed for more than 2 seconds
                        long_btnpress[MINI_BUTTON_B] = true;
                    }
                }
            }
            else if ((tC_BTN_LWR_RANGE_LIMIT < gadc_button_value) && (gadc_button_value < tC_BTN_UPR_RANGE_LIMIT))
            {
                // "C" button press detected
                if (miniBtnTimeDown[MINI_BUTTON_C] == 0)
                {  
                    miniButtonFalling[MINI_BUTTON_C] = true;
                    miniBtnTimeDown[MINI_BUTTON_C] = millis();
                }
                else
                {
                    tCurrentTime = millis();
                    if ((tCurrentTime - miniBtnTimeDown[MINI_BUTTON_C]) >= pdMS_TO_TICKS(BSP_LONG_PUSH_TIMEOUT_MS))
                    {
                        // "C" button kept pressed for more than 2 seconds
                        long_btnpress[MINI_BUTTON_C] = true;
                    }
                }
            }  
    }
}

static void send_key_value(void)
{
      //int key_detect_button = 4; // default
      int key_mode =  keyboard_mode_set.ghid_keyboard_mode;

      switch(key_mode)
      {
          case KEYBOARD_MODE_A:
          {
              if(keyboard_button_pressed[KEYBOARD_BUTTON_A])
              {
                  keyboard_button_pressed[KEYBOARD_BUTTON_A] = false;
                 // bsp_board_led_on(BLE_LED_NUM);
                  // send left arrow
              
              }

              else if (keyboard_button_pressed[KEYBOARD_BUTTON_C])
              {
                  keyboard_button_pressed[KEYBOARD_BUTTON_C] = false;
               //    bsp_board_led_off(BLE_LED_NUM);
                  // send right arrow
              
              }

              break;       
          }

          case KEYBOARD_MODE_B:
          {
          
             if(keyboard_button_pressed[KEYBOARD_BUTTON_A])
              {
                  keyboard_button_pressed[KEYBOARD_BUTTON_A] = false;
                  //bsp_board_led_on(BLE_LED_NUM);
                  // send up arrow
              
              }

              else if (keyboard_button_pressed[KEYBOARD_BUTTON_C])
              {
                  keyboard_button_pressed[KEYBOARD_BUTTON_C] = false;
                  //bsp_board_led_off(BLE_LED_NUM);
                  // send down arrow
              
              }

              break;          
          }
      
      
           case KEYBOARD_MODE_C:
          {
          
               if(keyboard_button_pressed[KEYBOARD_BUTTON_A])
              {
                  keyboard_button_pressed[KEYBOARD_BUTTON_A] = false;
                   // send page up
              
              }

              else if (keyboard_button_pressed[KEYBOARD_BUTTON_C])
              {
                  keyboard_button_pressed[KEYBOARD_BUTTON_C] = false;
                  // send page down
              }

              break;          
          }     
      }
}

void Process_Button_ABC (void)
{

    switch(xBLE_mode)
    {
        case MIDI_BLE_MODE:
        {
            for (byte x_index = 0; x_index < MINI_BUTTONS;  x_index++)
            {
                  // Hold Button for 2.5 seconds
                    if(button_long_detect[x_index])
                  {
                   // bsp_board_led_invert(BAT_RED_LED_NUM);
                    button_long_detect[x_index] = false;
                    g_sent_Notes(x_index + 4, 127, 1);
                  }

                  else if (miniButtonPressed[x_index])
                  {
                    miniButtonPressed[x_index] = false;
                    //bsp_board_led_invert(BLE_LED_NUM);
                    g_sent_Notes(x_index + 1, 127, 1);
                  }
            }
            break;
        }

        case KEYBOARD_BLE_MODE:
        {
        
          for (byte x_index = 0; x_index < MINI_BUTTONS;  x_index++)
          {
                
                if (miniButtonPressed[x_index])
                  {
                    miniButtonPressed[x_index] = false;
                    keyboard_button_pressed[x_index] = true;
                    //bsp_board_led_invert(BLE_LED_NUM);
                    send_key_value();
                  }
          }
           break;        
        }
    }
}

/**@brief thread for Looptimus function.
 *
 */
//void Process_Button_ABC (void);
void looptimus_mini_plus_app_thread(void * arg)
{
  UNUSED_PARAMETER(arg); 

  while (1)
  {
    {
        Check_BAT_Voltage();      // Enable when ADC circuit is connected

        Process_midi_Buttons();

        ext_pedal_process();

        Process_Button_ABC();
    }
        /* Delay a task for a given number of ticks */
        vTaskDelay(TASK_DELAY);
  }
} 

void Check_BAT_Voltage(void)
{
    int batvolt = 0;
    g_batvolt = 0;

    batvolt = measure_battery_volt();
    g_batvolt = batvolt;
    if(batvolt <= 671)    // 2.359 V on ADC pin / 3.008 Battery voltage
    {
        gBatLow = true;
        if (!gIsExternalPowerExists)
        {
            if (false == gLowBatVoltageIndication)
            {
                BatLowVoltage_StartTimer();
                gLowBatVoltageIndication = true;
            }
        }
        else
        {
            if (true == gLowBatVoltageIndication)
            {
                BatLowVoltage_StopTimer();
                gLowBatVoltageIndication = false;
            }            
        }
    }
   // else if(batvolt >= 677)   // 2.38 V on ADC pin / 3.035 Battery voltage
      else if(batvolt >= 677)
    {
        if (true == gLowBatVoltageIndication)
        {
            BatLowVoltage_StopTimer();
            gLowBatVoltageIndication = false;
        }
        gBatLow = false;        
    }
    else
    {
        //Do nothing
    }
}

/* @ function for power-on-sequence test

@description: This function configure diverse mode of looptimus mini plus at power on.

*/
bool gDelay(int delay_ms);
extern bool gtmr_3sec_Flag;
extern void start_delay_tmr(void);
extern bool gtmr_1min_Flag;

void Power_On_Sequence_Test(void)
{
    /*
    With Battery
    G: 460 to 711 > 1.6172 to 2.4996
    H: 220 to 345 > 0.7734 to 1.2129
    I: 114 to 180 > 0.4008 to 0.6328

    With external supply
    G: 460 + 320 to 711 + 140 > 780 to 851 > 2.7422 to 2.9918
    H: 220 + 175 to 345 +  85 > 395 to 430 > 1.3887 to 1.5117
    I: 114 +  62 to 180 +  40 > 176 to 220 > 0.6187 to 0.7734

    */
    // ADC Sampling on Battery 
    uint16_t tA_BTN_LWR_RANGE_LIMIT = 460, tA_BTN_UPR_RANGE_LIMIT = 711;
    uint16_t tB_BTN_LWR_RANGE_LIMIT = 220, tB_BTN_UPR_RANGE_LIMIT = 345;
    uint16_t tC_BTN_LWR_RANGE_LIMIT = 114, tC_BTN_UPR_RANGE_LIMIT = 180;

    uint16_t tAB_BTN_LWR_RANGE_LIMIT = 530, tAB_BTN_UPR_RANGE_LIMIT = 570;
    uint16_t tAC_BTN_LWR_RANGE_LIMIT = 510, tAC_BTN_UPR_RANGE_LIMIT = 540;
    uint16_t tBC_BTN_LWR_RANGE_LIMIT = 260, tBC_BTN_UPR_RANGE_LIMIT = 298;

    if(gIsExternalPowerExists)
    {
#ifdef  EXTERNAL_SUPPLY_IS_CONNECTED
            //ADC Sampling on External Power
            tA_BTN_LWR_RANGE_LIMIT += 320; tA_BTN_UPR_RANGE_LIMIT += 140;
            tB_BTN_LWR_RANGE_LIMIT += 175; tB_BTN_UPR_RANGE_LIMIT += 85;
            tC_BTN_LWR_RANGE_LIMIT += 62;  tC_BTN_UPR_RANGE_LIMIT += 40;

            tAB_BTN_LWR_RANGE_LIMIT += 130, tAB_BTN_UPR_RANGE_LIMIT += 125;
            tAC_BTN_LWR_RANGE_LIMIT += 120, tAC_BTN_UPR_RANGE_LIMIT += 120;
            tBC_BTN_LWR_RANGE_LIMIT += 70, tBC_BTN_UPR_RANGE_LIMIT += 70;
#endif
    }
        
    int16_t te1mp1;
    int16_t my_cnt = 0;

    int16_t t_cnt_idx, cnt;

    // clear midi flags.
    midi_flags.ccModeFlag = false;
    midi_flags.noteModeFlag = false;
    midi_flags.pcModeFlag = false;
    midi_flags.poc_test_done = false;
    midi_flags.rstModeFlag = false;

    // clear keyboard flags.
    keyboard_HID_flag.keyboard_mode_A_Flag = false;
    keyboard_HID_flag.keyboard_mode_B_Flag = false;
    keyboard_HID_flag.keyboard_mode_C_Flag = false;
    keyboard_HID_flag.hid_rstModeFlag = false;

    long x_my_time;
    long present_time;
    loop_variable.gread_button_value = 0;
    loop_variable.hold_time =  0;
    /* Read button status at Power on condition */
    
    loop_variable.gread_button_value = read_button_at_PoS();

    /* Power on sequence test for hold I and III when powering On */
/*
    if (643 <= btnStatus && btnStatus <= 657) //Target value = 650 (I & III)
    {
      dfuFlag = 1;
    }
*/

     switch(xBLE_mode)

     {  
        //MIDI Mode
        case MIDI_BLE_MODE:
        {

             if (tAB_BTN_LWR_RANGE_LIMIT <= loop_variable.gread_button_value && loop_variable.gread_button_value <= tAB_BTN_UPR_RANGE_LIMIT) // (I & II)
            {
                midi_flags.rstModeFlag = true;
            }
 
   
            else if (tBC_BTN_LWR_RANGE_LIMIT <= loop_variable.gread_button_value && loop_variable.gread_button_value <= tBC_BTN_UPR_RANGE_LIMIT) // (II & III)
            {
                midi_flags.expModeFlag = true;
            }
   
    
            else if (tA_BTN_LWR_RANGE_LIMIT <= loop_variable.gread_button_value && loop_variable.gread_button_value <= tA_BTN_UPR_RANGE_LIMIT) // (I)
            {
                midi_flags.noteModeFlag = true;
            }
   
            else if (tB_BTN_LWR_RANGE_LIMIT <= loop_variable.gread_button_value && loop_variable.gread_button_value <= tB_BTN_UPR_RANGE_LIMIT) //(II)
            {
                midi_flags.pcModeFlag = true;
            }
    
            else if (tC_BTN_LWR_RANGE_LIMIT <= loop_variable.gread_button_value && loop_variable.gread_button_value <= tC_BTN_UPR_RANGE_LIMIT) //(III)
            {
               midi_flags.ccModeFlag = true;
            }
            break;

        }
        
        //Keyboard MODE
        case KEYBOARD_BLE_MODE:
        {
            if (tAB_BTN_LWR_RANGE_LIMIT <= loop_variable.gread_button_value && loop_variable.gread_button_value <= tAB_BTN_UPR_RANGE_LIMIT) // (I & II)
            {
                keyboard_HID_flag.hid_rstModeFlag = true;
            }
    
            else if (tA_BTN_LWR_RANGE_LIMIT <= loop_variable.gread_button_value && loop_variable.gread_button_value <= tA_BTN_UPR_RANGE_LIMIT) // (I)
            {
                keyboard_HID_flag.keyboard_mode_A_Flag = true;
            }
   
            else if (tB_BTN_LWR_RANGE_LIMIT <= loop_variable.gread_button_value && loop_variable.gread_button_value <= tB_BTN_UPR_RANGE_LIMIT) //(II)
            {
                keyboard_HID_flag.keyboard_mode_B_Flag = true;
            }
    
            else if (tC_BTN_LWR_RANGE_LIMIT <= loop_variable.gread_button_value && loop_variable.gread_button_value <= tC_BTN_UPR_RANGE_LIMIT) //(III)
            {
               keyboard_HID_flag.keyboard_mode_C_Flag = true;
            }  
            break;
        }

     }

    /* ------------------------------------------------- Monitor button status for more than 3 seconds ----------------------------------*/

 /*
      uint16_t A_BTN_PRESSED_VALUE = 669, B_BTN_PRESSED_VALUE = 333, C_BTN_PRESSED_VALUE = 165,
                                     AB_BTN_PRESSED_VALUE = 548, AC_BTN_PRESSED_VALUE = 520, BC_BTN_PRESSED_VALUE = 278;
      
      if(gIsExternalPowerExists)
      {
          A_BTN_PRESSED_VALUE += 167, B_BTN_PRESSED_VALUE += 82, C_BTN_PRESSED_VALUE += 39
          AB_BTN_PRESSED_VALUE += 134, AC_BTN_PRESSED_VALUE += 128, BC_BTN_PRESSED_VALUE += 68;
      }
*/

/* ------------------------------------------------------- MIDI Power-On-Sequence Test  ----------------------------------------------*/

      // Hold Button (I & II) simultaneously for enough time to inter into memory mode
      if(midi_flags.rstModeFlag)
      {
        for(t_cnt_idx = 0; t_cnt_idx < 30; t_cnt_idx++)
        {
          if(loop_variable.gread_button_value < tAB_BTN_LWR_RANGE_LIMIT || loop_variable.gread_button_value > tAB_BTN_UPR_RANGE_LIMIT)
          {
              midi_flags.rstModeFlag = false;
              break;
          }
            nrf_delay_ms(100);
        }

        if(midi_flags.rstModeFlag)
        {
            // execute reset function
            BootModeRST();
        }
      }

      // Hold Button (II & III) simultaneously for enough time to inter into Expression Pedal Calibration Mode
      else if (midi_flags.expModeFlag)
      {
       
        for(t_cnt_idx = 0; t_cnt_idx < 30; t_cnt_idx++)
        {
          if(loop_variable.gread_button_value < tBC_BTN_LWR_RANGE_LIMIT || loop_variable.gread_button_value > tBC_BTN_UPR_RANGE_LIMIT)
          {
              midi_flags.expModeFlag = false;
              break;
          }
         // cnt = 10000;
         // while(cnt--);
            nrf_delay_ms(100);
        } 
        if(midi_flags.expModeFlag)
        {
            // EXPRESSION FUNCTION
            BootModeExp();
            //bsp_board_led_on(BLE_LED_NUM);
        }         
      }

      // Hold Button A for enough time to enter into NOTE Mode
      else if(midi_flags.noteModeFlag)
      {
        int8_t NoteMode_set = 0;

        for(t_cnt_idx = 0; t_cnt_idx < 30; t_cnt_idx++)
        {
          if(loop_variable.gread_button_value < tA_BTN_LWR_RANGE_LIMIT || loop_variable.gread_button_value > tA_BTN_UPR_RANGE_LIMIT)
          {
              midi_flags.noteModeFlag = false;
              break;
          }
            nrf_delay_ms(100);
        } 
        if((midi_flags.noteModeFlag) && (midi_mode_set.gmidi_mode != MIDI_MODE_NOTE))
        {
            midi_mode_set.gmidi_mode = MIDI_MODE_NOTE;
            // Save configuration in memory
            NoteMode_set = midi_mode_set.gmidi_mode + MODE_OFFSET_VAL;
            UpdateConfigData (PARAM_MODE,NO_SUB_PARAM,&NoteMode_set);
            //bsp_board_led_on(BLE_LED_NUM);
        }             
      }

      // Hold Button B for enough time to enter into PC Mode
      else if(midi_flags.pcModeFlag)
      {
        
        int8_t PCMode_set = 0;

        for(t_cnt_idx = 0; t_cnt_idx < 30; t_cnt_idx++)
        {
          if(loop_variable.gread_button_value < tB_BTN_LWR_RANGE_LIMIT || loop_variable.gread_button_value > tB_BTN_UPR_RANGE_LIMIT)
          {
              midi_flags.pcModeFlag = false;
              break;
          }
          
            nrf_delay_ms(100);
        } 
        if((midi_flags.pcModeFlag) && (midi_mode_set.gmidi_mode != MIDI_MODE_PC))
        {
            midi_mode_set.gmidi_mode = MIDI_MODE_PC;
            // Save configuration in memory
            PCMode_set = midi_mode_set.gmidi_mode + MODE_OFFSET_VAL;
            UpdateConfigData (PARAM_MODE,NO_SUB_PARAM,&PCMode_set);  
        }             
      }

      // Hold Button C for enough time to enter into CC Mode
      else if(midi_flags.ccModeFlag)
      {
        int8_t CCMode_set = 0;

        for(t_cnt_idx = 0; t_cnt_idx < 30; t_cnt_idx++)
        {
          if(loop_variable.gread_button_value < tC_BTN_LWR_RANGE_LIMIT || loop_variable.gread_button_value > tC_BTN_UPR_RANGE_LIMIT)
          {
              midi_flags.ccModeFlag = false;
              break;
          }

          nrf_delay_ms(100);
        } 
        if((midi_flags.ccModeFlag) && (midi_mode_set.gmidi_mode != MIDI_MODE_CC))
        {
            midi_mode_set.gmidi_mode = MIDI_MODE_CC;
            // Save configuration in memory
            CCMode_set = midi_mode_set.gmidi_mode + MODE_OFFSET_VAL;
            UpdateConfigData (PARAM_MODE,NO_SUB_PARAM,&CCMode_set);  
        }             
      }

/* ------------------------------------------------------- Keyboard Power-On-Sequence Test  ----------------------------------------------*/

      // Hold Button (I & II) simultaneously for enough time to inter into memory mode
      if(keyboard_HID_flag.hid_rstModeFlag)
      {
        for(t_cnt_idx = 0; t_cnt_idx < 30; t_cnt_idx++)
        {
          if(loop_variable.gread_button_value < tAB_BTN_LWR_RANGE_LIMIT || loop_variable.gread_button_value > tAB_BTN_UPR_RANGE_LIMIT)
          {
              keyboard_HID_flag.hid_rstModeFlag = false;
              break;
          }
            nrf_delay_ms(100);
        }

        if(keyboard_HID_flag.hid_rstModeFlag)
        {
            // execute reset function
            BootModeRST();
        }
      }

      else if (keyboard_HID_flag.keyboard_mode_A_Flag)
      {
           
        int8_t A_mode_set = 0;

        for(t_cnt_idx = 0; t_cnt_idx < 30; t_cnt_idx++)
        {
          if(loop_variable.gread_button_value < tA_BTN_LWR_RANGE_LIMIT || loop_variable.gread_button_value > tA_BTN_UPR_RANGE_LIMIT)
          {
              keyboard_HID_flag.keyboard_mode_A_Flag = false;
              break;
          }
            nrf_delay_ms(100);
        } 
        if((keyboard_HID_flag.keyboard_mode_A_Flag) && (keyboard_mode_set.ghid_keyboard_mode != KEYBOARD_MODE_A))
        {
            keyboard_mode_set.ghid_keyboard_mode = KEYBOARD_MODE_A;
            // Save configuration in memory
            A_mode_set = keyboard_mode_set.ghid_keyboard_mode;
            UpdateConfigData (PARAM_KEYBOARD_MODE,NO_SUB_PARAM,&A_mode_set);
            //bsp_board_led_on(BLE_LED_NUM);
        } 
      }
      
      else if (keyboard_HID_flag.keyboard_mode_B_Flag)
      {
           
        int8_t B_mode_set = 0;

        for(t_cnt_idx = 0; t_cnt_idx < 30; t_cnt_idx++)
        {
          if(loop_variable.gread_button_value < tB_BTN_LWR_RANGE_LIMIT || loop_variable.gread_button_value > tB_BTN_UPR_RANGE_LIMIT)
          {
              keyboard_HID_flag.keyboard_mode_B_Flag = false;
              break;
          }
            nrf_delay_ms(100);
        } 
        if((keyboard_HID_flag.keyboard_mode_B_Flag) && (keyboard_mode_set.ghid_keyboard_mode != KEYBOARD_MODE_B))
        {
            keyboard_mode_set.ghid_keyboard_mode = KEYBOARD_MODE_B;
            // Save configuration in memory
            B_mode_set = keyboard_mode_set.ghid_keyboard_mode;
            UpdateConfigData (PARAM_KEYBOARD_MODE,NO_SUB_PARAM,&B_mode_set);
            //bsp_board_led_on(BLE_LED_NUM);
        }       
      }

      else if (keyboard_HID_flag.keyboard_mode_C_Flag)
      {
           
        int8_t C_mode_set = 0;

        for(t_cnt_idx = 0; t_cnt_idx < 30; t_cnt_idx++)
        {
          if(loop_variable.gread_button_value < tC_BTN_LWR_RANGE_LIMIT || loop_variable.gread_button_value > tC_BTN_UPR_RANGE_LIMIT)
          {
              keyboard_HID_flag.keyboard_mode_C_Flag = false;
              break;
          }
            nrf_delay_ms(100);
        } 
        if((keyboard_HID_flag.keyboard_mode_C_Flag) && (keyboard_mode_set.ghid_keyboard_mode != KEYBOARD_MODE_C))
        {
            keyboard_mode_set.ghid_keyboard_mode = KEYBOARD_MODE_C;
            // Save configuration in memory
            C_mode_set = keyboard_mode_set.ghid_keyboard_mode;
            UpdateConfigData (PARAM_KEYBOARD_MODE,NO_SUB_PARAM,&C_mode_set);
            //bsp_board_led_on(BLE_LED_NUM);
        }       
      }
}

/* Set default MIDI mode */
void default_midi_mode (int8_t gmode_set)
{
  switch (gmode_set)
  {
    // set Note mode
    case MIDI_MODE_NOTE: 
    {
     // bsp_board_led_on(BLE_LED_NUM);
      midi_mode_set.gmidi_mode = MIDI_MODE_NOTE;
      break;
    }
    
    // set PC mode
    case MIDI_MODE_PC: 
    {
      midi_mode_set.gmidi_mode = MIDI_MODE_PC;
      break;
    }  
    
     // set CC mode
    case MIDI_MODE_CC: 
    {
//      bsp_board_led_on(BLE_LED_NUM);
      midi_mode_set.gmidi_mode = MIDI_MODE_CC;
      break;
    } 
  }
}


void init_looptimus(void)
{    
    midi_flags.detect_ext_pedal = false;

    // Set default MIDI mode
    default_midi_mode(MIDI_MODE_NOTE);

    // Initialize RT9471 
    RT9471_init ();
    
    //Initialize configuration
    Init_Configuration();

    //Read updated configuration
    ReadNUpdateConfigToDefault();
    
    //update midi mode
    if(gMode_midi <= 2)
    {
        midi_mode_set.gmidi_mode = gMode_midi;
    }

    // update hid keyboard mode
    if(gMode_HID_Keyboard <= 2)
    {
        keyboard_mode_set.ghid_keyboard_mode = gMode_HID_Keyboard;
    }
    // Carried - Out Power On Sequence test
    Power_On_Sequence_Test();
}


// Send MIDI notes 
void g_sent_Notes(byte x_note, byte y_velocity, byte z_channel)
{

   switch (midi_mode_set.gmidi_mode)
   {

    case MIDI_MODE_NOTE:
    {
        sendNoteOn(x_note,y_velocity,z_channel);
        sendNoteOff(x_note,0,z_channel);
        break;
    }

    case MIDI_MODE_PC:
    {
        sendProgramChange(x_note - 1,z_channel);
        break;
    }

    case MIDI_MODE_CC:
    {
      sendControlChange(x_note + 19,1,z_channel);
      break;
    }
   }
}

/* --------------------------------- External PEDAL Connected --------------------------------------*/
static inline byte RangeMap(int value, int oldRangeMin, int oldRangeMax, int newRangeMin, int newRangeMax)
{
    byte newValue = 0;

    newValue = ((float)((float)(value - oldRangeMin)/(float)(oldRangeMax - oldRangeMin)) * \
                    (newRangeMax - newRangeMin) + newRangeMin);

    return newValue;
}


void ext_pedal_process(void)
{
         if (firstTime > 100)
         {
              expRead = read_ext_pedal_value();
              expVal = digitalSmooth(expRead, expSmoothArray);
              expValMapped = RangeMap(expVal, expLow, expHigh, 0, 127);
              if(expValMapped > 127)
                  expValMapped = 127;
              else if(expValMapped < 0)
                  expValMapped = 0;

             if(lastExpVal != expVal && lastExpVal2 != expVal && lastExpValMapped != expValMapped)
                        
                {
                     sendControlChange(11, expValMapped, 1);  //11 = Expression Pedal
                     lastExpValMapped = expValMapped;
                     lastExpVal2 = lastExpVal;
                     //lastExpVal = expRead;
                     lastExpVal = expVal;
                }
                        //lastExpTime = millis();
         }
            
         else
         {
                 firstTime++;
         }
}


/*
void ext_pedal_process(void)
{
       tCurrentTime = millis();

       if((tCurrentTime - lastExpTime > pdMS_TO_TICKS(1)))
       {
              expRead = read_ext_pedal_value();
              expRead = read_ext_pedal_value();
              expVal = digitalSmooth(expRead, expSmoothArray);
              expValMapped = RangeMap(expVal, expLow, expHigh, 0, 127);

              if(expValMapped > 127)
                  expValMapped = 127;
              else if(expValMapped < 0)
                  expValMapped = 0;

             if(lastExpVal != expVal && lastExpVal2 != expVal && lastExpValMapped != expValMapped)
                        
                {
                     sendControlChange(11, expValMapped, 1);  //11 = Expression Pedal
                     lastExpValMapped = expValMapped;
                     lastExpVal2 = lastExpVal;
                    // lastExpVal = expRead;
                     lastExpVal = expVal;
                }
                lastExpTime = millis();
       }
        
}
*/

// "int *sensSmoothArray" passes an array to the function - the asterisk indicates the array name is a pointer
int digitalSmooth(int rawIn, int *sensSmoothArray)
{
    int j, k, temp, top, bottom;
    long total;
    static int i;
    // static int raw[filterSamples];
    static int sorted[filterSamples];
    boolean done;
    
    i = (i + 1) % filterSamples;    // increment counter and roll over if necc. -  % (modulo operator) rolls over variable
    sensSmoothArray[i] = rawIn;                 // input new data into the oldest slot
    
    // transfer data array into anther array for sorting and averaging
    for (j=0; j<filterSamples; j++)
    {
        sorted[j] = sensSmoothArray[j];
    }
    
    done = 0;                // flag to know when we're done sorting              
    while(done != 1){        // simple swap sort, sorts numbers from lowest to highest
        done = 1;
        for (j = 0; j < (filterSamples - 1); j++) {
            if (sorted[j] > sorted[j + 1]) {     // numbers are out of order - swap
                temp = sorted[j + 1];
                sorted [j + 1] =  sorted[j] ;
                sorted [j] = temp;
                done = 0;
            }
        }
    }
    
    // throw out top and bottom 15% of samples - limit to throw out at least one from top and bottom
    bottom = max(((filterSamples * 15)  / 100), 1); 
    top = min((((filterSamples * 85) / 100) + 1  ), (filterSamples - 1));   // the + 1 is to make up for asymmetry caused by integer rounding
    k = 0;
    total = 0;
    for ( j = bottom; j< top; j++){
      total += sorted[j];  // total remaining indices
      k++; 
    }
    
    return total / k;    // divide by number of samples
}

/* -------------------------------------------------------- Bootmode functions ----------------------------------------*/

/*
@ function:- BootModeRST (void)

  @description:- Reset flash memory and store reconfigured data.

*/
void BootModeRST(void)
{
    //initialize configuration
    Init_DefaultConfigData ();

    //Update configuration 
    UpdateAllConfigData (&gLooptimusConfigData);

    // Erase configuration data stored in memory
    EraseConfigData (); 
     
    // Write configuration data in memory   
    WriteConfigData ();     
}

/*
@ function:- BootModeExp (void)

  @description:- Expression Pedal Calibration at boot mode.

*/

uint32_t nrf_delay_ms_x(uint32_t ms_time);

void BootModeExp (void)
{
    int tPedalAdcVal = 0;
    int Xread_button = 0;
    int xLow_raw = 114, xHigh_raw = 711;
    int16_t cnt_ind = 0;
    bool button_press = false;
    bool button_release = false;
    uint32_t xcurrent_time = 0;
    uint32_t b_hold_time = 750; // 750ms

    if(gIsExternalPowerExists)
    {
        xLow_raw += 62, xHigh_raw+= 40;
    }

    for(;;)
    {
        tPedalAdcVal = read_ext_pedal_value();
        expVal = digitalSmooth(tPedalAdcVal, expSmoothArray);

        Xread_button = read_button_at_PoS();

        if(Xread_button < 100)
        {
            if((button_press) && (xcurrent_time >= b_hold_time))
            {
                  button_release = true;
                  button_press = false;
            }

        }

        else if (Xread_button > 100)
        {
            
            if((xLow_raw < Xread_button) && (xHigh_raw > Xread_button))
            {
                xcurrent_time = nrf_delay_ms_x(750);  // wait for some time and check again for release
                button_press = true;
            }
            
        }

        if(button_release)   //Next
        {
            nrf_delay_ms(100);  // wait for some time and check again for release
            if(miniButtonPressed[MINI_BUTTON_C] == false)   //Next
            button_release = false;
            if (!expNext)
            {
                //read expression pedal low value from ADC pin: AIN0
                tPedalAdcVal = read_ext_pedal_value();
                expLow = digitalSmooth(tPedalAdcVal, expSmoothArray);
                //expLow = AverageADC(m_buffer_pool); // Average ADC sample data
                if (expLow < 0)
                    expLow = 0;
                expNext = 1;
               // bsp_board_led_invert(BLE_LED_NUM);
            }
            else
            {
               // bsp_board_led_invert(BLE_LED_NUM);
                //read expression pedal high value from ADC pin: AIN0
                tPedalAdcVal = read_ext_pedal_value();
                expHigh = digitalSmooth(tPedalAdcVal, expSmoothArray);
                //expHigh = AverageADC(m_buffer_pool);    // Average ADC sample data
                expHigh = (expHigh - (expHigh / 127 / 2));
                // if data is valid
                if ((expHigh > expLow) && (expHigh > 0))
                {
                    //Update value in memory
                    UpdateConfigData(PARAM_EXP_PAD_CAL_LOW, NO_SUB_PARAM, &expLow);
                    UpdateConfigData(PARAM_EXP_PAD_CAL_HIGH, NO_SUB_PARAM, &expHigh);
                }
                else
                {
                    //read value from memory
                    GetParam_FromMemory(PARAM_EXP_PAD_CAL_LOW, NO_SUB_PARAM, &expLow);
                    GetParam_FromMemory(PARAM_EXP_PAD_CAL_HIGH, NO_SUB_PARAM, &expHigh);
                }
                break;
            }
        }  
    }
}
/*
void BootModeExp (void)
{
    int tPedalAdcVal = 0;

    for(;;)
    {
        tPedalAdcVal = read_ext_pedal_value();
        expVal = digitalSmooth(tPedalAdcVal, expSmoothArray);

        if(bsp_button_is_pressed(BUTTON_NEXT) == true)   //Next
        {
            nrf_delay_ms(100);  // wait for some time and check again for release
            if(bsp_button_is_pressed(BUTTON_NEXT) == false)   //Next

            if (!expNext)
            {
                //read expression pedal low value from ADC pin: AIN0
                TakeSample(expPin);                 // Start taking ADC data
                while (vIsAdcConvDone == false);     // wait while sampling is going on
                tPedalAdcVal = AverageADC(m_buffer_pool); // Average ADC sample data
                expLow = digitalSmooth(tPedalAdcVal, expSmoothArray);
                //expLow = AverageADC(m_buffer_pool); // Average ADC sample data
                if (expLow < 0)
                    expLow = 0;
                expNext = 1;
                DisplayExpPedal (expNext);
            }
            else
            {
                //read expression pedal high value from ADC pin: AIN0
                TakeSample(expPin);                     // Start taking ADC data
                while (vIsAdcConvDone == false);         // wait while sampling is going on
                tPedalAdcVal = AverageADC(m_buffer_pool); // Average ADC sample data
                expHigh = digitalSmooth(tPedalAdcVal, expSmoothArray);
                //expHigh = AverageADC(m_buffer_pool);    // Average ADC sample data
                expHigh = (expHigh - (expHigh / 127 / 2));
                Display3Dash();
                // if data is valid
                if ((expHigh > expLow) && (expHigh > 0))
                {
                    //Update value in memory
                    UpdateConfigData(PARAM_EXP_PAD_CAL_LOW, NO_SUB_PARAM, &expLow);
                    UpdateConfigData(PARAM_EXP_PAD_CAL_HIGH, NO_SUB_PARAM, &expHigh);
                }
                else
                {
                    //read value from memory
                    GetParam_FromMemory(PARAM_EXP_PAD_CAL_LOW, NO_SUB_PARAM, &expLow);
                    GetParam_FromMemory(PARAM_EXP_PAD_CAL_HIGH, NO_SUB_PARAM, &expHigh);
                }
                DisplayNote(mode, 0);
                break;
            }
        }
        displayDigits();   
    }
}
*/