/** @file
 *
 * @defgroup looptimus_plus flashWrite.c
 * @{
 * @brief    Write the configuration data into Flash.
 *
 * This file contains the source code to store the various configuration data
 *  into internal Flash memory.
 * 
 */

#ifndef FLASH_WRITE_H__
#define FLASH_WRITE_H__

#define MEM_OFFSET_EXP_PEDAL_CAL  1
#define MEM_OFFSET_MODE_CHNL      2
#define MEM_OFFSET_PCMAP          11
#define MEM_OFFSET_KEYBOARD_BOND  3
#define MEM_OFFSET_KEYBOARD_MODE  4
#define MEM_OFFSET_EXTMODE_MAP    35
#ifdef CC_MODE_SUPPORT
#define MEM_OFFSET_CCMAP          36
#define MEM_OFFSET_EXTMODECC_MAP  68
#endif      //#ifdef CC_MODE_SUPPORT

#define FW_VER_MEMORY_ADDR       0x6D000

void Config_Memory_init(void);
void ReadConfigData (void);
void WriteConfigData (void);
void EraseConfigData (void);
void Flash_PartialPage_Erase (void);

unsigned short int Read8Bits_FromMemory (int offset, int pos);
unsigned short int Read16Bits_FromMemory (int offset, int pos);
unsigned short int Read32Bits_FromMemory (int offset);

#endif  //FLASH_WRITE_H__