/** @file
 *
 * @defgroup looptimus_plus looptimus_app.c
 * @{
 * @brief    main functionality of Looptimus Plus device.
 *
 * This file contains the source code for application that looptimus plus
 * device requires to do.
 * 
 */

#ifndef LOOPTIMUS_MINI_PLUS_APP_H__
#define LOOPTIMUS_MINI_PLUS_APP_H__

//#include "definition.h"
#include "MIDI.h"

typedef unsigned char byte;
typedef unsigned char boolean;

#define false 0
#define true 1
/*-------------- LOOPTIMUS MINI PLUS DEFINITION -----------*/

#define  BLE_KEYBOARD_MODE_DETECT                      1
#define  BLE_MIDI_MODE_DETECT                          2

#define DEF_CHANNEL_NUM   1

#define MAX_BANK          21
#define NOTES_PER_BANK    6
#define ONBOARD_BUTTONS   9
#define MINI_BUTTONS      3

#define MIN_CHANNEL_NUM   1
#define MAX_CHANNEL_NUM   16
#define DEF_CHANNEL_NUM   1

#define MIN_OCTAVE_VAL    (signed int)-3
#define MAX_OCTAVE_VAL    3
#define DEF_OCTAVE_VAL    0

#define EXPLOWDEF   0     //Default low end cutoff
#define EXPHIGHDEF  938   //Default high end cutoff

#define MODE_OFFSET_VAL   128
#define NOTE_OFFSET_VAL   128
#define OCTAVE_OFFSET_VAL   4

#define MAX_BANK          21


#define TOTAL_BUTTONS     3 

/*BLE MODE */
typedef enum
{
  MIDI_BLE_MODE = 0,
  KEYBOARD_BLE_MODE = 1,
}BLE_MODE_SELECT;

typedef struct 
{
  int8_t gmidi_mode;
}MAIN_MIDI_MODE_SET;

typedef struct
{
  int8_t ghid_keyboard_mode;
}MAIN_HID_KEYBOARD_MODE_SET;

typedef enum
{
  KEYBOARD_BUTTON_A = 0,
  KEYBOARD_BUTTON_B = 1,
  KEYBOARD_BUTTON_C = 2,
}KEYBOARD_BUTTONS;

typedef enum {
    MODE_DEF = 0,
    MODE_PCC = 1,
    MODE_PAD = 2,
#ifdef CC_MODE_SUPPORT
    MODE_CC = 3,
    MAX_MODES = 4
#else
    MAX_MODES = 3
#endif      //#ifdef CC_MODE_SUPPORT
} modeName_t;

typedef enum {
    EXTMODE_GHI = 0,
    EXTMODE_NT = 1,
    EXTMODE_PC = 2,
    EXTMODE_CTL = 3,
    EXTMODE_CHL = 4,
#ifdef CC_MODE_SUPPORT
    EXTMODE_CC = 5,
    MAX_EXTMODE = 6
#else
    MAX_EXTMODE = 5
#endif
} extModes_t;

/* Looptimus Mini+ MIDI Mode defined */
typedef enum
{
  KEYBOARD_MODE_A = 0,
  KEYBOARD_MODE_B = 1,
  KEYBOARD_MODE_C = 2,
} keyboard_mode_defined;

/* Looptimus Mini+ HID Keyboard Mode defined */
typedef enum
{
  MIDI_MODE_NOTE = 0,
  MIDI_MODE_PC = 1,
  MIDI_MODE_CC = 2,
} midi_mode_defined;

extern int expLow, expHigh;
extern byte mode;
extern byte curBank[], curChan[];
extern int8_t octave;
extern byte ledBrightness;
extern byte pcMap[], pcMapA[];
extern byte ccMap[], ccMapA[];
extern byte extMode;
extern boolean extExists;
extern byte tSwVer[4];


// Set the flag whenever any parameter is updated and needs to write into Flash
extern boolean gUpdateFlashData;
extern boolean gEraseConfigData_f;

/*----------------- DEFINITION ----------------------*/
#define MINI_BUTTONS      3

#define MINI_BUTTON_A   0
#define MINI_BUTTON_B    1
#define MINI_BUTTON_C    2
/*------- DEFINITION - SAADC --------------*/
#define SAMPLES_IN_BUFFER 4

/************* USER-DEFINED FUNCTIONS --------------*/
void Check_button_status(int gadc_button_value);

#endif