#include "nrf_gpio.h"
#include "definition.h"
#include "cmsis_gcc.h"


#define BAT_I2C_SET_SCL    nrf_gpio_pin_set(BAT_SCL_PIN);
#define BAT_I2C_CLR_SCL    nrf_gpio_pin_clear(BAT_SCL_PIN);
#define BAT_I2C_SET_SDA    nrf_gpio_pin_set(BAT_SDA_PIN);
#define BAT_I2C_CLR_SDA    nrf_gpio_pin_clear(BAT_SDA_PIN);

#define BAT_I2C_INPORT     nrf_gpio_pin_read(BAT_SDA_PIN)

#define BAT_I2C_DELAY       user_delay();

void user_delay(void);

void user_delay(void)
{
    __NOP();
    __NOP();
    __NOP();
    __NOP();
    __NOP();
    __NOP();
    __NOP();
    __NOP();
    __NOP();
    __NOP();
    __NOP();
    __NOP();
    __NOP();
    __NOP();
    __NOP();
    __NOP();
    __NOP();
    __NOP();
}


void bat_i2c_init(void)
{
    BAT_I2C_SET_SCL
    BAT_I2C_SET_SDA

    BAT_I2C_DELAY
}


//Initiating an I2C start condition:
void bat_i2c_start_condition( void )
{
    BAT_I2C_SET_SCL
    BAT_I2C_SET_SDA
    BAT_I2C_DELAY
    BAT_I2C_CLR_SDA
    BAT_I2C_DELAY
    BAT_I2C_CLR_SCL
    BAT_I2C_DELAY
}


// Initiating an I2C stop condition:
void bat_i2c_stop_condition( void )
{
    BAT_I2C_CLR_SDA
    BAT_I2C_DELAY
    BAT_I2C_SET_SCL
    BAT_I2C_DELAY
    BAT_I2C_SET_SDA
    BAT_I2C_DELAY
}


// Writing a bit in I2C:
void bat_i2c_write_bit( uint8_t b )
{
    if( b > 0 )
        BAT_I2C_SET_SDA
    else
        BAT_I2C_CLR_SDA

    BAT_I2C_DELAY
    BAT_I2C_SET_SCL
    BAT_I2C_DELAY
    BAT_I2C_CLR_SCL
}


// Reading a bit in I2C:
uint8_t bat_i2c_read_bit( void )
{
    uint8_t b;
     
    BAT_I2C_DELAY
    BAT_I2C_SET_SCL

    if( BAT_I2C_INPORT )
    {
        b = 1;
    }
    else
    {
        b = 0;
    }
    
    BAT_I2C_CLR_SCL

    return b;
}


// Writing a byte with I2C:
bool bat_i2c_write_byte( uint8_t B,
                     bool start,
                     bool stop )
{
    uint8_t ack = 0;

    if( start )
        bat_i2c_start_condition();

    uint8_t i;
    for( i = 0; i < 8; i++ )
    {
        bat_i2c_write_bit( B & 0x80 );   // write the most-significant bit
        B <<= 1;
    }
    
    ack = bat_i2c_read_bit();

    if( stop )
        bat_i2c_stop_condition();
    
    return !ack;
}


// Reading a byte with I2C:
uint8_t bat_i2c_read_byte( bool ack,
                       bool stop )
{
    uint8_t B = 0;

    uint8_t i;
    
    nrf_gpio_pin_dir_set (BAT_SDA_PIN,  NRF_GPIO_PIN_DIR_INPUT);

    for( i = 0; i < 8; i++ )
    {
        B <<= 1;
        B |= bat_i2c_read_bit();
    }

    nrf_gpio_pin_dir_set (BAT_SDA_PIN,  NRF_GPIO_PIN_DIR_OUTPUT);

//  Master Acknowledment
    if( ack )
        bat_i2c_write_bit(0);
    else
        bat_i2c_write_bit(1);
////////////////////////
    if( stop )
        bat_i2c_stop_condition();


    return B;
}


// Sending a byte with I2C:
bool bat_i2c_send_byte( uint8_t address,
                    uint8_t data )
{
    if( bat_i2c_write_byte( address << 1, true, false ) )   // start, send address, write
    {
        // send data, stop
        if( bat_i2c_write_byte( data, false, true ) )
            return true;
    }
    
    bat_i2c_stop_condition();   // make sure to impose a stop if NAK'd
    return false;
}


// Receiving a byte with a I2C:
uint8_t bat_i2c_receive_byte( uint8_t address )
{
    if( bat_i2c_write_byte( ( address << 1 ) | 0x01, true, false ) )   // start, send address, read
    {
        return bat_i2c_read_byte( false, true );
    }

    return 0;   // return zero if NAK'd
}

// Sending a byte of data with I2C:
bool bat_i2c_send_byte_data( uint8_t address,
                         uint8_t reg,
                         uint8_t data )
{
    if( bat_i2c_write_byte( address << 1, true, false ) )   // start, send address, write
    {
        if( bat_i2c_write_byte( reg, false, false ) )   // send desired register
        {
            if( bat_i2c_write_byte( data, false, true ) ) return true;   // send data, stop
        }
    }

    bat_i2c_stop_condition();
    return false;
}

// Receiving a byte of data with I2C:
uint8_t bat_i2c_receive_byte_data( uint8_t address,
                               uint8_t reg )
{
    if( bat_i2c_write_byte( address << 1, true, false ) )   // start, send address, write
    {
        if( bat_i2c_write_byte( reg, false, false ) )   // send desired register
        {
            if( bat_i2c_write_byte( ( address << 1) | 0x01, true, false ) )   // start again, send address, read
            {
                return bat_i2c_read_byte( false, true );   // read data
            }
        }
    }

    bat_i2c_stop_condition();
    return 0;   // return zero if NACKed
}