/** @file
 *
 * @defgroup looptimus_plus looptimus_config.c
 * @{
 * @brief    configuration data of Looptimus Plus device.
 *
 * This file contains the source code for configuration that looptimus plus
 * device runs with.
 * 
 */
 
#include <stdio.h>
#include <stdint.h>

#include "looptimus_mini_plus_app.h"
#include "looptimus_config.h"
#include "MemoryOpr_sd.h"

#define OCTBRGT_GRP_NUM 31

#define KEYBOARD_DEFAULT_WRITE                  2   // Default mode is advertising
#define KEYBOARD_BLANK_READ                     255
#define KEYBOARD_BLANK_READ_ZERO                     0
looptimus_config_data_t gLooptimusConfigData;

// Set the flag whenever any parameter is updated and needs to write into Flash
boolean gUpdateFlashData = false;
boolean gEraseConfigData_f = false;
extern int xKeyboard_bond_state;
byte gMode_midi = 0;
byte gMode_HID_Keyboard = 0;
boolean first_flash_keyboard;
extern int xBLE_mode;
int Read8Bits_Int_FromMemory (int offset, int pos);

extern int8_t config_midi_mode;
extern int8_t config_keyboard_mode;
byte MemRead_PCCMapChnl (byte channel);
byte MemRead_ExtPCMapChnl (byte channel);
#ifdef CC_MODE_SUPPORT
byte MemRead_CCMapChnl (byte channel);
byte MemRead_ExtCCMapChnl (byte channel);
#endif

/*
void CheckNCorrectNotesForPCMode (byte temp)
{
    if ((temp >= NOTE_OFFSET_VAL) && (temp <= 0xfd))  //All map values should be 128 or greater, if initialized
    {
        //Read all Memory PC map values
        for (byte i = 0; i < (MAX_BANK * NOTES_PER_BANK); i++)
        {
             pcMap[i] = ( (MemRead_PCCMapChnl (i)) & 0x7F);
        }
    }
    else
    {
        //Initialize Program Change mappings to Memory
        for (byte i = 0; i < (MAX_BANK * NOTES_PER_BANK); i++)
        {
            temp = (pcMap[i] + NOTE_OFFSET_VAL);
            UpdateConfigData (PARAM_PCC_MAP, i, &temp); // 20-Oct
        }
    }
  
    temp = 0;
    GetParam_FromMemory(PARAM_EXT_PAD_PCC, 0, &temp);
    if ((temp >= NOTE_OFFSET_VAL) && (temp <= 0xFF))  //All map values should be 128 or greater, if initialized
    {
        for (byte i = 0; i < MAX_MODES; i++)
        {
            pcMapA[i] = ( (MemRead_ExtPCMapChnl (i)) & 0x7F);
        }
    }
    else
    {
      for (byte i = 0; i < 3; i++)
      {
          temp = (pcMapA[i] + NOTE_OFFSET_VAL);
          UpdateConfigData (PARAM_EXT_PAD_PCC, i, &temp);
      }
    }
}
*/
#ifdef CC_MODE_SUPPORT
void CheckNCorrectNotesForCCMode (byte temp)
{
    if ((temp >= NOTE_OFFSET_VAL) && (temp <= 0xfd))  //All map values should be 128 or greater, if initialized
    {
        //Read all Memory PC map values
        for (byte i = 0; i < (MAX_BANK * NOTES_PER_BANK); i++)
        {
             ccMap[i] = ( (MemRead_CCMapChnl (i)) & 0x7F);
        }
    }
    else
    {
        //Initialize Program Change mappings to Memory
        for (byte i = 0; i < (MAX_BANK * NOTES_PER_BANK); i++)
        {
            temp = (ccMap[i] + NOTE_OFFSET_VAL);
            UpdateConfigData (PARAM_CC_MAP, i, &temp); // 20-Oct
        }
    }
  
    temp = 0;
    GetParam_FromMemory(PARAM_EXT_PAD_CC, 0, &temp);
    if ((temp >= NOTE_OFFSET_VAL) && (temp <= 0xFF))  //All map values should be 128 or greater, if initialized
    {
        for (byte i = 0; i < MAX_MODES; i++)
        {
            ccMapA[i] = ( (MemRead_ExtCCMapChnl (i)) & 0x7F);
        }
    }
    else
    {
      for (byte i = 0; i < 3; i++)
      {
          temp = (ccMapA[i] + NOTE_OFFSET_VAL);
          UpdateConfigData (PARAM_EXT_PAD_CC, i, &temp);
      }
    }
}
#endif      //#ifdef CC_MODE_SUPPORT
/*
void CheckNCorrectChannels (void)
{
    GetParam_FromMemory(PARAM_DEFAULT_MODE_CHNL, NO_SUB_PARAM, &curChan[MODE_DEF]);
    GetParam_FromMemory(PARAM_PCC_MODE_CHNL, NO_SUB_PARAM, &curChan[MODE_PCC]);
    GetParam_FromMemory(PARAM_PAD_MODE_CHNL, NO_SUB_PARAM, &curChan[MODE_PAD]);
#ifdef CC_MODE_SUPPORT
    GetParam_FromMemory(PARAM_CC_MODE_CHNL, NO_SUB_PARAM, &curChan[MODE_CC]);
#endif  //#ifdef CC_MODE_SUPPORT
    // Correct channels if out of range
    if ((curChan[MODE_DEF] < MIN_CHANNEL_NUM) || (curChan[MODE_DEF] > MAX_CHANNEL_NUM))
    {
        curChan[MODE_DEF] = DEF_CHANNEL_NUM;
        UpdateConfigData (PARAM_DEFAULT_MODE_CHNL, NO_SUB_PARAM, &curChan[MODE_DEF]);
    }

    if ((curChan[MODE_PCC] < MIN_CHANNEL_NUM) || (curChan[MODE_PCC] > MAX_CHANNEL_NUM))
    {
        curChan[MODE_PCC] = DEF_CHANNEL_NUM;
        UpdateConfigData (PARAM_PCC_MODE_CHNL, NO_SUB_PARAM, &curChan[MODE_PCC]);
    }

    if ((curChan[MODE_PAD] < MIN_CHANNEL_NUM) || (curChan[MODE_PAD] > MAX_CHANNEL_NUM))
    {
        curChan[MODE_PAD] = DEF_CHANNEL_NUM;
        UpdateConfigData (PARAM_PAD_MODE_CHNL, NO_SUB_PARAM, &curChan[MODE_PAD]);
    }
#ifdef CC_MODE_SUPPORT
    if ((curChan[MODE_CC] < MIN_CHANNEL_NUM) || (curChan[MODE_CC] > MAX_CHANNEL_NUM))
    {
        curChan[MODE_CC] = DEF_CHANNEL_NUM;
        UpdateConfigData (PARAM_CC_MODE_CHNL, NO_SUB_PARAM, &curChan[MODE_CC]);
    }
#endif      //#ifdef CC_MODE_SUPPORT
}
*/

void Init_DefaultConfigData (void)
{
   // Set Pedal Low Value
   expLow = EXPLOWDEF;
   // Set Pedal High Value
   expHigh = EXPHIGHDEF;
   
   //Configure Defualt MiDi Mode
   config_midi_mode = MODE_OFFSET_VAL; 
   
   config_keyboard_mode = 0; // default set in Keyboard_Mode_A

   // config ble keyboard bond build mode
   xKeyboard_bond_state = 1;
#ifdef DESABLE_ME   
   curChan [MODE_DEF] = DEF_CHANNEL_NUM;
   curChan [MODE_PCC] = DEF_CHANNEL_NUM;
   curChan [MODE_PAD] = DEF_CHANNEL_NUM;
#ifdef CC_MODE_SUPPORT
   curChan [MODE_CC] = DEF_CHANNEL_NUM;
#endif
   octave = DEF_OCTAVE_VAL;
   
   //ledBrightness = DEF_BRIGHTNESS;

   extMode = EXTMODE_GHI;

   for (byte i = 0; i < (MAX_BANK * NOTES_PER_BANK); i++)
   {
       pcMap[i] = i;
#ifdef CC_MODE_SUPPORT
       ccMap[i] = i;
#endif
   }

   for (byte i = 0; i < MINI_BUTTONS; i++)
   {
       pcMapA[i] = (i + 125);
#ifdef CC_MODE_SUPPORT
       ccMapA[i] = (i + 125);
#endif
   }
#endif
}

/**
 * @brief Function for application main entry.
 */

void Init_Configuration(void)
{
    Config_Memory_init();

    // Update configuration data to default values
    Init_DefaultConfigData ();

    // Update structure having configuration data
    UpdateAllConfigData (&gLooptimusConfigData);
    // Read configuration data from memory
    ReadConfigData ();
}

void UpdateAllConfigData (looptimus_config_data_t *pConfigData)
{

    // Update Expression Pedal Calibration value
    pConfigData->mExpPedalCalibration = ((((uint32_t)expHigh << 16) & 0xFFFF0000) |
                                          ((uint16_t)expLow));

    // Update MIDI Mode
    pConfigData ->mMode_midi = (((uint32_t)config_midi_mode << 24) & 0xFF000000);

    // Update BLE Keyboard Bond
    pConfigData ->keyboard_bond_state = (((uint32_t)xKeyboard_bond_state << 24) & 0xFF000000);

    // Update BLE Keyboard Mode
    pConfigData ->mMode_keyboard = (((uint32_t)config_keyboard_mode << 24) & 0xFF000000);

#ifdef CC_MODE_SUPPORT
    for (i = 0, j = 0; (i + 3) < (MAX_BANK * NOTES_PER_BANK); j++, i = i + 4)
    {
        pConfigData->mCCMap[j] = ( (((uint32_t)(ccMap[i + 3] + NOTE_OFFSET_VAL) << 24) & 0xFF000000) | 
                                 (((uint32_t)(ccMap[i + 2] + NOTE_OFFSET_VAL) << 16) & 0xFF0000) | 
                                 (((uint32_t)(ccMap[i + 1] + NOTE_OFFSET_VAL) << 8) & 0xFF00) | 
                                 (ccMap[i] + NOTE_OFFSET_VAL) );
    }
    // Last 32-bit word
    if (i < (MAX_BANK * NOTES_PER_BANK))
    {
        pConfigData->mCCMap[j] = ( (((uint32_t)(0xFF) << 24) & 0xFF000000) | 
                                 (((uint32_t)(0xFF) << 16) & 0xFF0000) | 
                                   (((uint32_t)(ccMap[i + 1] + NOTE_OFFSET_VAL) << 8) & 0xFF00) | 
                                               (ccMap[i] + NOTE_OFFSET_VAL) );
    }

    pConfigData->mExtModeCCMapChnl = ( (((uint32_t)(curChan[MODE_CC]) << 24) & 0xFF000000) | 
                                 (((uint32_t)(ccMapA[2] + NOTE_OFFSET_VAL) << 16) & 0xFF0000) | 
                                 (((uint32_t)(ccMapA[1] + NOTE_OFFSET_VAL) << 8) & 0xFF00) | 
                                 (ccMapA[0] + NOTE_OFFSET_VAL));
#endif      //#ifdef CC_MODE_SUPPORT
}

byte t4ChnlGrpNum;
byte temp;

void UpdateConfigData (byte paramNumber, byte arg, void *value)
{
    byte updateFlag = 1;        // TODO: Set to 1 to write configuration in memory

    switch (paramNumber)
    {
      default:
        updateFlag = 0;
        break;

      case PARAM_EXP_PAD_CAL_LOW:
        gLooptimusConfigData.mExpPedalCalibration &= 0xFFFF0000;  //Reset the value to 0
        gLooptimusConfigData.mExpPedalCalibration |= (*(uint16_t *)value);
        break;

      case PARAM_EXP_PAD_CAL_HIGH:
        gLooptimusConfigData.mExpPedalCalibration &= 0x0000FFFF;  //Reset the value to 0
        gLooptimusConfigData.mExpPedalCalibration |= ( (*(uint16_t *)value) << 16);
        break;

      case PARAM_MODE:
        //Reset the value to 0
        gLooptimusConfigData.mMode_midi &= (uint32_t)(~(0xFF << (MODE_VAL_POS * 8)));
        gLooptimusConfigData.mMode_midi |= ( (*(byte *)value) << (MODE_VAL_POS * 8));
        break;

      case PARAM_KEYBOARD_BOND:
        gLooptimusConfigData.keyboard_bond_state &= (uint32_t)(~(0xFF << (MODE_VAL_POS * 8)));  //Reset the value to 0
        gLooptimusConfigData.keyboard_bond_state |=((*(byte *)value) << (MODE_VAL_POS * 8));
        break;

      case PARAM_KEYBOARD_MODE:
        gLooptimusConfigData.mMode_keyboard &= (uint32_t)(~(0xFF << (MODE_VAL_POS * 8)));
        gLooptimusConfigData.mMode_keyboard |=  ( (*(byte *)value) << (MODE_VAL_POS * 8));
        break;

#ifdef CC_MODE_SUPPORT
      case PARAM_CC_MAP:
      {
        byte t4ChnlGrpNum = (arg / 4);
        byte temp = ((arg % 4) * 8);
        //Reset the value to 0
        gLooptimusConfigData.mCCMap[t4ChnlGrpNum] &= (0xFFFFFFFF & (uint32_t)(~(0xFF << temp)));
        gLooptimusConfigData.mCCMap[t4ChnlGrpNum] |= ((*(byte *)value) << temp);
      }  break;

      case PARAM_EXT_PAD_CC:
      {
        byte temp = ((arg % 4) * 8);
        //Reset the value to 0
        gLooptimusConfigData.mExtModeCCMapChnl &= (0xFFFFFFFF & (uint32_t)(~(0xFF << temp)));
        gLooptimusConfigData.mExtModeCCMapChnl |= ((*(byte *)value) << temp);
      }  break;

      case PARAM_CC_MODE_CHNL:
        gLooptimusConfigData.mExtModeCCMapChnl &= 0x00FFFFFF;  //Reset the value to 0 for lower nibble
        gLooptimusConfigData.mExtModeCCMapChnl |= ( ( (*(byte *)value)) << (CC_MODE_CHNL_VAL_POS * 8));
        break;
#endif      //#ifdef CC_MODE_SUPPORT
    }

    if (updateFlag)
    {
        gUpdateFlashData = true;
    }
}
byte gTempVar = 0;

#define  DESABLE_ME

void ReadNUpdateConfigToDefault (void)
{
#ifndef DESABLE_ME
    //Initialize Program Change Mappings
    for (byte i = 0; i < (MAX_BANK * NOTES_PER_BANK); i++)
    {
        pcMap[i] = i;
    }
    for (byte i = 0; i < MAX_MODES; i++)
    {
        pcMapA[i] = (i + 125);
    }
#endif

    GetParam_FromMemory(PARAM_KEYBOARD_BOND,NO_SUB_PARAM, &xKeyboard_bond_state);

    if((!xKeyboard_bond_state) || (xKeyboard_bond_state == 255) )
    {
        first_flash_keyboard = true;
        xKeyboard_bond_state = 1;
        UpdateConfigData (PARAM_KEYBOARD_BOND,NO_SUB_PARAM,&xKeyboard_bond_state);
    }

   if(xBLE_mode == MIDI_BLE_MODE)
   {
          // Read the expression pedal calibration settings from memory
          GetParam_FromMemory (PARAM_EXP_PAD_CAL_LOW, NO_SUB_PARAM, &expLow);
          GetParam_FromMemory (PARAM_EXP_PAD_CAL_HIGH, NO_SUB_PARAM, &expHigh);

          //If calibration settings do not exist, write the default values to memory
          if ((expHigh == EXT_PEDAL_0V_ADC_VAL) || (expHigh > EXT_PEDAL_3V3_ADC_VAL))
          {
              expLow = EXPLOWDEF;
              expHigh = EXPHIGHDEF;
              UpdateConfigData (PARAM_EXP_PAD_CAL_LOW, NO_SUB_PARAM, &expLow);
              UpdateConfigData (PARAM_EXP_PAD_CAL_HIGH, NO_SUB_PARAM, &expHigh);
          }
          //Read the mode flag from memory
    
          byte temp = 0;
          GetParam_FromMemory (PARAM_MODE, NO_SUB_PARAM, &temp);

          // Check memory for last saved mode
          if ((temp >= MODE_OFFSET_VAL) && (temp < (MODE_OFFSET_VAL + MAX_MODES)))
          {
              gMode_midi = (temp - MODE_OFFSET_VAL);
          }
          else          //Memory has not been initialized, turn PC mode off (default)
          {
              temp = MODE_OFFSET_VAL;
              UpdateConfigData (PARAM_MODE, NO_SUB_PARAM,  &temp);
          }

   }
   
   else if (xBLE_mode == KEYBOARD_BLE_MODE)
   {
     
        byte temp = 0;
        GetParam_FromMemory (PARAM_KEYBOARD_MODE, NO_SUB_PARAM, &temp); 
          
        // Check memory for last saved mode
        if ((temp >= 0) && (temp < 3))
          {
              gMode_HID_Keyboard = temp;
          }   
            
        //Memory has not been initialized
        else   
        {
              temp = 0;
              UpdateConfigData (PARAM_KEYBOARD_MODE, NO_SUB_PARAM,  &temp);
        }     
     
   }

   else
   {
       // nothing
   }

#ifndef DESABLE_ME
  // Read current channel from Memory for each mode
    CheckNCorrectChannels ();
  //Read current octave from Memory
    octave = DEF_OCTAVE_VAL;
    GetParam_FromMemory(PARAM_OCTAVE, NO_SUB_PARAM, &gTempVar);
    if (gTempVar > 0)
        temp = gTempVar;
    if ((temp < (MIN_OCTAVE_VAL + OCTAVE_OFFSET_VAL)) || 
        (temp > (MAX_OCTAVE_VAL + OCTAVE_OFFSET_VAL)))
    {
        octave = DEF_OCTAVE_VAL;
        temp = (octave + OCTAVE_OFFSET_VAL);
        UpdateConfigData (PARAM_OCTAVE, NO_SUB_PARAM, &temp);
    }
    else
    {
        octave = (int8_t)(temp - OCTAVE_OFFSET_VAL);
    }
   
    //Read LED Brightness from Memory
   // ledBrightness = DEF_BRIGHTNESS;
    GetParam_FromMemory(PARAM_BRIGHTNESS, NO_SUB_PARAM, &ledBrightness);
    //Correct brightness if out of range
    /*
    if ((ledBrightness < MIN_BRIGHTNESS) || (ledBrightness > MAX_BRIGHTNESS))
    {
        ledBrightness = DEF_BRIGHTNESS;
        UpdateConfigData (PARAM_BRIGHTNESS, NO_SUB_PARAM, &ledBrightness);
    }
  */
    //Update_Brightness(ledBrightness);
    //Read Extension pedal mode from Memory
    extMode = 0;
    GetParam_FromMemory(PARAM_EXT_PAD_MODE, NO_SUB_PARAM, &extMode);
    if ((extMode < 0) || (extMode > MAX_EXTMODE))
    {
        extMode = EXTMODE_GHI;
    }

    //Read the first PC map value
    //**TODO** - Expand mappings for up to 3 expansion buttons
    temp = NOTE_OFFSET_VAL;
    GetParam_FromMemory(PARAM_PCC_MAP, 0, &temp);   // 20-Oct
    CheckNCorrectNotesForPCMode (temp);
#endif

#ifdef CC_MODE_SUPPORT
    //Read the first CC map value
    //**TODO** - Expand mappings for up to 3 expansion buttons
    temp = NOTE_OFFSET_VAL;
    GetParam_FromMemory(PARAM_CC_MAP, 0, &temp);   // 20-Oct
    CheckNCorrectNotesForCCMode (temp);
#endif      //#ifdef CC_MODE_SUPPORT
}
/*
Expression pedal calibration value: low bytes
*/
unsigned short int MemRead_ExpPadCalLow (void)
{
    return (Read16Bits_FromMemory (MEM_OFFSET_EXP_PEDAL_CAL, EXP_PEDAL_CAL_LOW_POS));
}

/*
Expression pedal calibration value: high bytes
*/
unsigned short int MemRead_ExpPadCalHigh (void)
{
    return (Read16Bits_FromMemory (MEM_OFFSET_EXP_PEDAL_CAL, EXP_PEDAL_CAL_HIGH_POS));
}

/*
Mode value: possible values: mode + 128
*/
byte MemRead_Mode (void)
{
    return (Read8Bits_FromMemory (MEM_OFFSET_EXP_PEDAL_CAL, MODE_VAL_POS));
}

/*

/*
Range: 1 to 16
*/
static byte MemRead_DefaultModeChnl (void)
{
    return (Read8Bits_FromMemory (MEM_OFFSET_MODE_CHNL, DEF_CHNL_VAL_POS));
}

/*
Range: 1 to 16
*/
static byte MemRead_PCCModeChnl (void)
{
    return (Read8Bits_FromMemory (MEM_OFFSET_PCMAP, PCC_CHNL_VAL_POS));
}

//keyboard bond state
static byte MemRead_keyboard (void)
{
    return (Read8Bits_FromMemory (MEM_OFFSET_KEYBOARD_BOND, MODE_VAL_POS));
}

//keyboard mode
static byte MemRead_keyboard_Mode (void)
{
    return (Read8Bits_FromMemory (MEM_OFFSET_KEYBOARD_MODE, MODE_VAL_POS));
}
/*
Range: 1 to 16
*/
static byte MemRead_PADModeChnl (void)
{
    return (Read8Bits_FromMemory (MEM_OFFSET_MODE_CHNL, PAD_CHNL_VAL_POS));
}

#ifdef CC_MODE_SUPPORT
/*
Range: 1 to 16
*/
static byte MemRead_CCModeChnl (void)
{
    return (Read8Bits_FromMemory (MEM_OFFSET_EXTMODECC_MAP, CC_MODE_CHNL_VAL_POS));
}
#endif      //#ifdef CC_MODE_SUPPORT
/*
Range: 1 to 16
*/
byte Mem_ReadCurChannel (byte mode)
{
    byte tCurChannel = (MAX_CHANNEL_NUM + 1);
    switch (mode)
    {
        default:
            break;

        case MODE_DEF:
            tCurChannel = MemRead_DefaultModeChnl ();
            break;

        case MODE_PCC:
            tCurChannel = MemRead_PCCModeChnl();
            break;

#ifdef CC_MODE_SUPPORT
        case MODE_CC:
            tCurChannel = MemRead_CCModeChnl();
            break;
#endif
        case MODE_PAD:
            tCurChannel = MemRead_PADModeChnl();
            break;
    }

    return tCurChannel;
}

/*
Range: 0 to 6
*/
byte uint32Offset = OCTBRGT_GRP_NUM;
byte channelValue = 0;

byte MemRead_Octave (void)
{
    channelValue = (byte)(Read8Bits_FromMemory (MEM_OFFSET_PCMAP + uint32Offset, OCTAVE_VAL_POS));

    return channelValue;
}

/*
Range: 1 to 7
*/

byte MemRead_Brightness (void)
{
    uint32Offset = OCTBRGT_GRP_NUM;

    channelValue = (byte)(Read8Bits_FromMemory (MEM_OFFSET_PCMAP + uint32Offset, BRIGHTNETSS_VAL_POS));

    return channelValue;
}

byte MemRead_ExtPadMode (void)
{
    return (Read8Bits_FromMemory (MEM_OFFSET_EXTMODE_MAP, EXTMODE_VAL_POS));
}
uint32_t tempPCCMapVal = 0;
/*
channel : starting from 128 to 255
*/
byte MemRead_PCCMapChnl (byte channel)
{
    //byte uint32Offset = (channel / 4);
    //byte channelValue = 0;
    uint32Offset = (channel / 4);
    channelValue = 0;

    //channelValue = ((Read32Bits_FromMemory (MEM_OFFSET_PCMAP + uint32Offset)) >> ((channel % 4) * 8));

    tempPCCMapVal = Read32Bits_FromMemory (MEM_OFFSET_PCMAP + uint32Offset);

    uint32Offset = ((channel % 4) * 8);
    channelValue = (tempPCCMapVal >> uint32Offset);

    return channelValue;
}

/*
channel : starting from 253, 254 and 255
*/
byte MemRead_ExtPCMapChnl (byte channel)
{
    //byte uint32Offset = (channel / 4);
    //byte channelValue = 0;
    uint32Offset = (channel / 4);
    channelValue = 0;

    //channelValue = ((Read32Bits_FromMemory (MEM_OFFSET_EXTMODE_MAP + uint32Offset)) >> ((channel % 4) * 8));

    tempPCCMapVal = Read32Bits_FromMemory (MEM_OFFSET_EXTMODE_MAP + uint32Offset);

    uint32Offset = ((channel % 4) * 8);
    channelValue = (tempPCCMapVal >> uint32Offset);

    return channelValue;

    return ((Read8Bits_FromMemory (MEM_OFFSET_EXTMODE_MAP, channel)) );
}

#ifdef CC_MODE_SUPPORT
/*
channel : starting from 128 to 255
*/
byte MemRead_CCMapChnl (byte channel)
{
    uint32Offset = (channel / 4);
    channelValue = 0;

    tempPCCMapVal = Read32Bits_FromMemory (MEM_OFFSET_CCMAP + uint32Offset);

    uint32Offset = ((channel % 4) * 8);
    channelValue = (tempPCCMapVal >> uint32Offset);

    return channelValue;
}

/*
channel : starting from 253, 254 and 255
*/
byte MemRead_ExtCCMapChnl (byte channel)
{
    uint32Offset = ((channel / 4) % 32);
    channelValue = 0;

    tempPCCMapVal = Read32Bits_FromMemory (MEM_OFFSET_EXTMODECC_MAP + uint32Offset);

    uint32Offset = ((channel % 4) * 8);
    channelValue = (tempPCCMapVal >> uint32Offset);

    return channelValue;
}
#endif      //#ifdef CC_MODE_SUPPORT

void GetParam_FromMemory (byte paramNumber, byte arg, void *value)
{
    switch (paramNumber)
    {
      default:
        break;

      case PARAM_EXP_PAD_CAL_LOW:
        *(unsigned short int*)value = MemRead_ExpPadCalLow();
        break;

      case PARAM_EXP_PAD_CAL_HIGH:
        *(unsigned short int*)value = MemRead_ExpPadCalHigh();
        break;

      case PARAM_MODE:
        *(byte *)value = MemRead_Mode();
        break;

      case PARAM_KEYBOARD_BOND:
        *(int *)value = MemRead_keyboard();
        break;

      case PARAM_KEYBOARD_MODE:
        *(byte *)value = MemRead_keyboard_Mode();

#ifndef DESABLE_ME
      case PARAM_DEFAULT_MODE_CHNL:
        *(byte *)value = Mem_ReadCurChannel (MODE_DEF);
        break;

      case PARAM_PCC_MODE_CHNL:
        *(byte *)value = Mem_ReadCurChannel (MODE_PCC);
        break;

      case PARAM_PAD_MODE_CHNL:
        *(byte *)value = Mem_ReadCurChannel (MODE_PAD);
        break;
#endif
#ifdef CC_MODE_SUPPORT
      case PARAM_CC_MODE_CHNL:
        *(byte *)value = Mem_ReadCurChannel (MODE_CC);
        break;
#endif
#ifndef DESABLE_ME
      case PARAM_OCTAVE:
        *(byte *)value = MemRead_Octave();
        break;

      case PARAM_BRIGHTNESS:
        *(byte *)value = MemRead_Brightness();
        break;

      case PARAM_PCC_MAP:
        *(byte *)value = MemRead_PCCMapChnl(arg);
        break;
#endif
#ifdef CC_MODE_SUPPORT
      case PARAM_CC_MAP:
        *(byte *)value = MemRead_CCMapChnl(arg);
        break;
#endif
#ifndef DESABLE_ME
      case PARAM_EXT_PAD_MODE:
        *(byte *)value = MemRead_ExtPadMode();
        break;

      case PARAM_EXT_PAD_PCC:
        *(byte *)value = MemRead_ExtPCMapChnl(arg);
        break;
#endif
#ifdef CC_MODE_SUPPORT
      case PARAM_EXT_PAD_CC:
        *(byte *)value = MemRead_ExtCCMapChnl(arg);
        break;
#endif
    }
}