/** @file
 *
 * @defgroup looptimus_mini_plus user_timer.c
 * @{
 * @brief    initialization of timer functions.
 *
 * This file contains the source code for timers
 */

#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>
#include <string.h>

#include "definition.h"
#include "nrf_drv_clock.h"
#include "sdk_errors.h"
#include "app_error.h"

#define  BATTERY_TIMER_INIT

// Below value is in seconds
#define MAX_WAIT_B4_FLASH_WRITE     5

#ifndef BLE_MIDI_SUPPORT
#define MAX_PARTIAL_PAGE_COUNT      9
#endif

extern void BatOperation_Timer_Handler (void *p_context);
void BatLowVoltage_Timer_Handler (void *p_context);
void init_delay_tmr (uint32_t delay_time);
void ble_timer_init (void);

int tmr_cnt = 0;
bool long_time_detect = false;

int gtimer_cnt = 0;
int g_tmr_3s = 0;
int gtmr_1sc = 0;
int gtmr_1min = 0;
int gtmr_1hr =0;


bool gtmr_1min_Flag = false;
bool gtmr_1sec_Flag = false;
bool gtmr_1hr_Flag = false;
bool gtmr_3sec_Flag = false;

APP_TIMER_DEF(m_batoperation_timer);
APP_TIMER_DEF(m_batlowvoltage_timer);
APP_TIMER_DEF(m_delay_ms);
APP_TIMER_DEF(m_flash_write_timer);

byte sWaitCountB4Write = 0;
extern int xBLE_mode;
void FlashWrite_StartTimer (void)
{
    sWaitCountB4Write = 0;
    app_timer_start (m_flash_write_timer, APP_TIMER_TICKS(FLASH_WRITE_TIMER_INTERVAL), NULL);
}

void FlashWrite_StopTimer (void)
{
    app_timer_stop (m_flash_write_timer);
}

void BatLowVoltage_Timer_Handler (void *p_context)
{
    //bsp_board_led_invert(BAT_RED_LED_NUM);
}

void g_delay_ms(void *p_context)
{
    gtimer_cnt++; // ISR at every 100ms
    g_tmr_3s++;
    //1 second
    if(gtimer_cnt > 10)
    {
      gtmr_1sc++;
      gtimer_cnt = 0;
      gtmr_1min_Flag = true;
    }
  
    //1 second
    if(g_tmr_3s > 30)
    {
      gtmr_3sec_Flag = true;
    }

    // 1 min
    if (gtmr_1sc >= 60)
    {
      gtmr_1sc = 0;
      gtmr_1min++;
      gtmr_1sec_Flag = true;
    }

    // 1 hr
    if(gtmr_1min >= 60)
    {
      gtmr_1min = 0;
      gtmr_1hr++;
      gtmr_1hr_Flag = true;

    }
}

void BatOperation_StartTimer (void)
{
    app_timer_start (m_batoperation_timer, APP_TIMER_TICKS(BAT_OPERATION_INTERVAL), NULL);

    tmr_cnt++;
    if (tmr_cnt >= 3)
    {
      tmr_cnt = 0;
      long_time_detect = true;
    }

}

void BatOperation_StopTimer (void)
{
    app_timer_stop (m_batoperation_timer);
}

void BatLowVoltage_StartTimer(void)
{
  app_timer_start (m_batlowvoltage_timer, APP_TIMER_TICKS(LOW_BAT_BLINK_INTERVAL), NULL);
}

void BatLowVoltage_StopTimer(void)
{
  app_timer_stop (m_batlowvoltage_timer);
}

void flash_write_timer_handler (void * p_context)
{
    sWaitCountB4Write++;

    if (sWaitCountB4Write >= MAX_WAIT_B4_FLASH_WRITE)
    {
        FlashWrite_StopTimer ();
        gUpdateFlashData = false;
        WriteConfigData ();
        sWaitCountB4Write = 0;
    }
}

void user_timer_init(void)
{
    ret_code_t err_code = app_timer_init();
    APP_ERROR_CHECK(err_code);
#ifdef BLE_MIDI_SUPPORT
    if(xBLE_mode == MIDI_BLE_MODE)
    {
       ble_timer_init();
    }

    else
    {
          // nothing
    }
   
#endif

#ifndef BLE_MIDI_SUPPORT
    err_code = app_timer_create(&m_flash_erase, APP_TIMER_MODE_REPEATED, partial_page_erase_handler);
    APP_ERROR_CHECK(err_code);
#endif      //#ifndef BLE_MIDI_SUPPORT

    err_code = app_timer_create(&m_flash_write_timer, APP_TIMER_MODE_REPEATED, flash_write_timer_handler);
    APP_ERROR_CHECK(err_code);

#ifdef  BATTERY_TIMER_INIT
    err_code = app_timer_create(&m_batoperation_timer, APP_TIMER_MODE_REPEATED, BatOperation_Timer_Handler);
    APP_ERROR_CHECK(err_code);

    err_code = app_timer_create(&m_batlowvoltage_timer, APP_TIMER_MODE_REPEATED, BatLowVoltage_Timer_Handler);
    APP_ERROR_CHECK(err_code);

    err_code = app_timer_create(&m_delay_ms, APP_TIMER_MODE_REPEATED, g_delay_ms);
    APP_ERROR_CHECK(err_code);

#endif

}

void start_delay_tmr(void)
{

      app_timer_start (m_delay_ms, APP_TIMER_TICKS(DELAY_TIMER_MS), NULL);

}

/*
bool gDelay(int delay_ms)
{
    //iteration of 100ms
    start_delay_tmr();
    for (int xd = 0; xd > delay_ms; xd++)
    {
      gtmr_1sc
    
    }
    //while((delay_ms - gtimer_cnt) != 0); 
   // app_timer_stop(m_delay_ms);
    return true;
}
*/

void delay_event_handler(nrf_timer_event_t event_type, void* p_context)
{
    static uint32_t i;
    uint32_t led_to_invert = ((i++) % LEDS_NUMBER);

    switch (event_type)
    {
        case NRF_TIMER_EVENT_COMPARE0:
            bsp_board_led_invert(led_to_invert);
            break;

        default:
            //Do nothing.
            break;
    }
}

uint32_t nrf_delay_ms_x(uint32_t ms_time)
{
    int xtick_indx = 0;

    if (ms_time == 0)
    {
        return 0;
    }

    do {
        nrf_delay_us(1000);
        xtick_indx++;
    } while (--ms_time);
    return xtick_indx;
}